import unittest

import mox

import hyou.util


class LazyOrderedDictionaryTest(unittest.TestCase):
  def setUp(self):
    self.mox = mox.Mox()
    self.enumerator = self.mox.CreateMockAnything()
    self.constructor = self.mox.CreateMockAnything()
    self.dict = hyou.util.LazyOrderedDictionary(
        enumerator=self.enumerator, constructor=self.constructor)

  def tearDown(self):
    self.mox.UnsetStubs()
    self.mox.VerifyAll()

  def test_enumerate(self):
    self.enumerator().AndReturn(
        [('A', 'apple'), ('B', 'banana'), ('C', 'cinamon')])
    self.mox.ReplayAll()

    # iter()
    it = iter(self.dict)
    self.assertEqual('A', it.next())
    self.assertEqual('B', it.next())
    self.assertEqual('C', it.next())
    self.assertRaises(StopIteration, it.next)
    # len()
    self.assertEqual(3, len(self.dict))
    # keys()
    self.assertEqual(['A', 'B', 'C'], self.dict.keys())
    # values()
    self.assertEqual(['apple', 'banana', 'cinamon'], self.dict.values())
    # items()
    self.assertEqual(
        [('A', 'apple'), ('B', 'banana'), ('C', 'cinamon')],
        self.dict.items())
    # Indexing by an integer
    self.assertEqual('apple', self.dict[0])
    self.assertEqual('banana', self.dict[1])
    self.assertEqual('cinamon', self.dict[2])
    # Indexing by a key
    self.assertEqual('apple', self.dict['A'])
    self.assertEqual('banana', self.dict['B'])
    self.assertEqual('cinamon', self.dict['C'])

  def test_construct(self):
    self.constructor('A').AndReturn('apple')
    self.mox.ReplayAll()
    self.assertEqual('apple', self.dict['A'])

  def test_refresh(self):
    self.constructor('A').AndReturn('apple1')
    self.constructor('A').AndReturn('apple2')
    self.mox.ReplayAll()

    self.dict.refresh()
    self.assertEqual('apple1', self.dict['A'])
    self.assertEqual('apple1', self.dict['A'])
    self.dict.refresh()
    self.assertEqual('apple2', self.dict['A'])
    self.assertEqual('apple2', self.dict['A'])
    self.dict.refresh()

  def test_construct_then_enumerate(self):
    self.constructor('B').AndReturn('bacon')
    self.enumerator().AndReturn(
        [('A', 'apple'), ('B', 'banana'), ('C', 'cinamon')])
    self.mox.ReplayAll()

    self.assertEqual('bacon', self.dict['B'])
    self.assertEqual(['A', 'B', 'C'], self.dict.keys())
    self.assertEqual(['apple', 'bacon', 'cinamon'], self.dict.values())

  def test_enumerate_then_construct_unlisted(self):
    self.enumerator().AndReturn([('A', 'apple'), ('C', 'cinamon')])
    self.constructor('B').AndReturn('banana')
    self.mox.ReplayAll()

    self.assertEqual(['A', 'C'], self.dict.keys())
    self.assertEqual(['apple', 'cinamon'], self.dict.values())
    self.assertEqual('banana', self.dict['B'])
    self.assertEqual(['A', 'C', 'B'], self.dict.keys())
    self.assertEqual(['apple', 'cinamon', 'banana'], self.dict.values())

  def test_construct_then_enumerate_unlisted(self):
    self.constructor('B').AndReturn('banana')
    self.enumerator().AndReturn([('A', 'apple'), ('C', 'cinamon')])
    self.mox.ReplayAll()

    self.assertEqual('banana', self.dict['B'])
    self.assertEqual(['A', 'C', 'B'], self.dict.keys())
    self.assertEqual(['apple', 'cinamon', 'banana'], self.dict.values())

  def test_no_constructor_indexing(self):
    self.constructor('A').AndReturn(None)
    self.enumerator().AndReturn([('A', 'apple')])
    self.mox.ReplayAll()
    self.assertEqual('apple', self.dict['A'])

  def test_no_constructor_indexing_miss(self):
    self.constructor('A').AndReturn(None)
    self.enumerator().AndReturn([('B', 'banana')])
    self.mox.ReplayAll()
    self.assertRaises(KeyError, self.dict.__getitem__, 'A')

  def test_get(self):
    self.enumerator().AndReturn([('A', 'apple')])
    self.constructor('B').AndReturn(None)
    self.mox.ReplayAll()
    list(self.dict)
    self.assertEqual('apple', self.dict.get('A', 'missing'))
    self.assertEqual('missing', self.dict.get('B', 'missing'))


class PseudoList(hyou.util.CustomMutableFixedList):
  def __init__(self, real_list):
    self.real_list = real_list

  def __getitem__(self, i):
    return self.real_list[i]

  def __iter__(self):
    return iter(self.real_list)

  def __len__(self):
    return len(self.real_list)

  def __setitem__(self, i, value):
    self.real_list[i] = value


class CustomMutableFixedListTest(unittest.TestCase):
  def setUp(self):
    self.list = PseudoList(['apple', 'banana', 'cinamon', 'apple'])

  def test_ordering(self):
    self.assertEqual(['apple', 'banana', 'cinamon', 'apple'], self.list)
    self.assertNotEqual(['apple', 'banana', 'cinamon', 'lemon'], self.list)
    self.assertNotEqual(['apple', 'banana', 'cinamon'], self.list)
    self.assertLess(['apple', 'bacon', 'cinamon', 'apple'], self.list)
    self.assertGreater(['apple', 'lemon', 'cinamon', 'apple'], self.list)
    self.assertLess(['apple', 'banana', 'cinamon'], self.list)
    self.assertGreater(
        ['apple', 'banana', 'cinamon', 'apple', 'lemon'], self.list)

  def test_contains(self):
    self.assertTrue('apple' in self.list)
    self.assertFalse('bacon' in self.list)

  def test_index(self):
    self.assertEqual(0, self.list.index('apple'))
    self.assertRaises(ValueError, self.list.index, 'bacon')

  def test_count(self):
    self.assertEqual(2, self.list.count('apple'))
    self.assertEqual(1, self.list.count('banana'))
    self.assertEqual(0, self.list.count('bacon'))

  def test_reverse(self):
    self.list.reverse()
    self.assertEqual('apple', self.list[0])
    self.assertEqual('cinamon', self.list[1])
    self.assertEqual('banana', self.list[2])
    self.assertEqual('apple', self.list[3])

  def test_sort(self):
    self.list.sort()
    self.assertEqual('apple', self.list[0])
    self.assertEqual('apple', self.list[1])
    self.assertEqual('banana', self.list[2])
    self.assertEqual('cinamon', self.list[3])

  def test_unsupported(self):
    self.assertRaises(NotImplementedError, self.list.__delitem__, 0)
    self.assertRaises(NotImplementedError, self.list.append, 'lemon')
    self.assertRaises(NotImplementedError, self.list.extend, ['lemon'])
    self.assertRaises(NotImplementedError, self.list.insert, 0, 'lemon')
    self.assertRaises(NotImplementedError, self.list.pop)
    self.assertRaises(NotImplementedError, self.list.remove, 'apple')



import datetime
import unittest

import gdata.spreadsheets.client
import mox

import hyou.client


class FakeSpreadsheetTitleFeed(object):
  def __init__(self, title):
    self.text = title


class FakeSpreadsheetUpdatedFeed(object):
  def __init__(self):
    self.text = '2015-08-11T12:34:56.789Z'


class FakeSpreadsheetFeed(object):
  def __init__(self, title):
    self.title = FakeSpreadsheetTitleFeed(title)
    self.updated = FakeSpreadsheetUpdatedFeed()


class FakeWorksheetFeed(object):
  def __init__(self, key):
    self._key = key

  def get_worksheet_id(self):
    return self._key


class FakeWorksheetsFeed(object):
  def __init__(self, entries):
    self.entry = entries


class SpreadsheetTest(unittest.TestCase):
  def setUp(self):
    self.mox = mox.Mox()
    self.mox.StubOutClassWithMocks(hyou.client, 'Worksheet')
    self.client = self.mox.CreateMock(
        gdata.spreadsheets.client.SpreadsheetsClient)
    self.drive = self.mox.CreateMockAnything()
    entry = FakeSpreadsheetFeed('Cinamon')
    self.spreadsheet = hyou.client.Spreadsheet(
        None, self.client, self.drive, 'cinamon', entry)

  def tearDown(self):
    self.mox.UnsetStubs()
    self.mox.VerifyAll()

  def set_enumerator_expectations(self):
    sheet1_feed = FakeWorksheetFeed('s1')
    sheet2_feed = FakeWorksheetFeed('s2')
    sheet3_feed = FakeWorksheetFeed('s3')
    feed = FakeWorksheetsFeed([sheet1_feed, sheet2_feed, sheet3_feed])

    self.client.get_worksheets('cinamon').AndReturn(feed)
    sheet1 = hyou.client.Worksheet(
        self.spreadsheet, self.client, 's1', sheet1_feed)
    sheet1.key = 's1'
    sheet1.title = 'Sheet1'
    sheet2 = hyou.client.Worksheet(
        self.spreadsheet, self.client, 's2', sheet2_feed)
    sheet2.key = 's2'
    sheet2.title = 'Sheet2'
    sheet3 = hyou.client.Worksheet(
        self.spreadsheet, self.client, 's3', sheet3_feed)
    sheet3.key = 's3'
    sheet3.title = 'Sheet3'
    return (sheet1, sheet2, sheet3)

  def set_refresh_expectations(self):
    self.client.get_feed(
        hyou.client.SPREADSHEET_URL % 'cinamon',
        desired_class=gdata.spreadsheets.data.Spreadsheet)

  def test_worksheet_accessors(self):
    sheet1, sheet2, sheet3 = self.set_enumerator_expectations()
    self.mox.ReplayAll()

    # iter()
    it = iter(self.spreadsheet)
    self.assertEqual('Sheet1', it.next())
    self.assertEqual('Sheet2', it.next())
    self.assertEqual('Sheet3', it.next())
    self.assertRaises(StopIteration, it.next)
    # len()
    self.assertEqual(3, len(self.spreadsheet))
    # keys()
    self.assertEqual(['Sheet1', 'Sheet2', 'Sheet3'], self.spreadsheet.keys())
    # values()
    self.assertEqual([sheet1, sheet2, sheet3], self.spreadsheet.values())
    # items()
    self.assertEqual(
        [('Sheet1', sheet1), ('Sheet2', sheet2), ('Sheet3', sheet3)],
        self.spreadsheet.items())
    # Indexing by an integer
    self.assertEqual(sheet1, self.spreadsheet[0])
    self.assertEqual(sheet2, self.spreadsheet[1])
    self.assertEqual(sheet3, self.spreadsheet[2])
    # Indexing by a key
    self.assertEqual(sheet1, self.spreadsheet['Sheet1'])
    self.assertEqual(sheet2, self.spreadsheet['Sheet2'])
    self.assertEqual(sheet3, self.spreadsheet['Sheet3'])

  def test_refresh(self):
    self.set_refresh_expectations()

    self.mox.ReplayAll()

    self.spreadsheet.refresh()

  def test_add_worksheet(self):
    self.client.add_worksheet('cinamon', 'Sheet3', rows=2, cols=8)
    self.set_refresh_expectations()
    sheet1, sheet2, sheet3 = self.set_enumerator_expectations()

    self.mox.ReplayAll()

    self.assertEqual(
        sheet3, self.spreadsheet.add_worksheet('Sheet3', rows=2, cols=8))

  def test_delete_worksheet(self):
    self.set_enumerator_expectations()
    self.client.delete(
        gdata.spreadsheets.client.WORKSHEET_URL % ('cinamon', 's3'),
        force=True)
    self.set_refresh_expectations()

    self.mox.ReplayAll()

    self.spreadsheet.delete_worksheet('Sheet3')

  def test_url(self):
    self.assertEqual(
        'https://docs.google.com/spreadsheets/d/cinamon/edit',
        self.spreadsheet.url)

  def test_title(self):
    self.assertEqual('Cinamon', self.spreadsheet.title)

  def test_title_setter(self):
    files = self.mox.CreateMockAnything()
    self.drive.files().AndReturn(files)
    executor = self.mox.CreateMockAnything()
    files.update(fileId='cinamon', body={'title': 'Lemon'}).AndReturn(executor)
    executor.execute().AndReturn({})
    self.set_refresh_expectations()

    self.mox.ReplayAll()

    self.spreadsheet.title = 'Lemon'

  def test_updated(self):
    self.assertEqual(
        datetime.datetime(
            year=2015, month=8, day=11, hour=12, minute=34, second=56,
            microsecond=789000),
        self.spreadsheet.updated)



import unittest

import gdata.spreadsheets.client
import mox

import hyou.client


class FakeSpreadsheetFeed(object):
  def __init__(self, key):
    self._key = key

  def get_spreadsheet_key(self):
    return self._key


class FakeSpreadsheetsFeed(object):
  def __init__(self, entries):
    self.entry = entries


class CollectionTest(unittest.TestCase):
  def setUp(self):
    self.mox = mox.Mox()
    self.mox.StubOutClassWithMocks(hyou.client, 'Spreadsheet')
    self.client = self.mox.CreateMock(
        gdata.spreadsheets.client.SpreadsheetsClient)
    self.drive = self.mox.CreateMockAnything()
    self.collection = hyou.client.Collection(self.client, self.drive)

  def tearDown(self):
    self.mox.UnsetStubs()
    self.mox.VerifyAll()

  def test_accessors_with_constructor(self):
    banana_feed = FakeSpreadsheetFeed('banana')
    self.client.get_feed(
        hyou.client.SPREADSHEET_URL % 'banana',
        desired_class=gdata.spreadsheets.data.Spreadsheet
    ).AndReturn(banana_feed)
    banana = hyou.client.Spreadsheet(
        self.collection, self.client, self.drive, 'banana', banana_feed)
    self.mox.ReplayAll()

    # Indexing by a key
    self.assertEqual(banana, self.collection['banana'])

  def test_accessors_with_enumerator(self):
    apple_feed = FakeSpreadsheetFeed('apple')
    banana_feed = FakeSpreadsheetFeed('banana')
    cinamon_feed = FakeSpreadsheetFeed('cinamon')
    feed = FakeSpreadsheetsFeed([apple_feed, banana_feed, cinamon_feed])

    self.client.get_spreadsheets().AndReturn(feed)
    apple = hyou.client.Spreadsheet(
        self.collection, self.client, self.drive, 'apple', apple_feed)
    banana = hyou.client.Spreadsheet(
        self.collection, self.client, self.drive, 'banana', banana_feed)
    cinamon = hyou.client.Spreadsheet(
        self.collection, self.client, self.drive, 'cinamon', cinamon_feed)

    self.mox.ReplayAll()

    # iter()
    it = iter(self.collection)
    self.assertEqual('apple', it.next())
    self.assertEqual('banana', it.next())
    self.assertEqual('cinamon', it.next())
    self.assertRaises(StopIteration, it.next)
    # len()
    self.assertEqual(3, len(self.collection))
    # keys()
    self.assertEqual(['apple', 'banana', 'cinamon'], self.collection.keys())
    # values()
    self.assertEqual([apple, banana, cinamon], self.collection.values())
    # items()
    self.assertEqual(
        [('apple', apple), ('banana', banana), ('cinamon', cinamon)],
        self.collection.items())
    # Indexing by an integer
    self.assertEqual(apple, self.collection[0])
    self.assertEqual(banana, self.collection[1])
    self.assertEqual(cinamon, self.collection[2])
    # Indexing by a key
    self.assertEqual(apple, self.collection['apple'])
    self.assertEqual(banana, self.collection['banana'])
    self.assertEqual(cinamon, self.collection['cinamon'])

  def test_create_spreadsheet(self):
    files = self.mox.CreateMockAnything()
    self.drive.files().AndReturn(files)
    executor = self.mox.CreateMockAnything()
    files.insert(body={
        'title': 'Cinamon',
        'mimeType': 'application/vnd.google-apps.spreadsheet',
    }).AndReturn(executor)
    executor.execute().AndReturn({'id': 'cinamon'})

    cinamon_feed = FakeSpreadsheetFeed('cinamon')
    self.client.get_feed(
        hyou.client.SPREADSHEET_URL % 'cinamon',
        desired_class=gdata.spreadsheets.data.Spreadsheet
    ).AndReturn(cinamon_feed)
    cinamon = hyou.client.Spreadsheet(
        self.collection, self.client, self.drive, 'cinamon', cinamon_feed)

    worksheet = self.mox.CreateMockAnything()
    cinamon[0].AndReturn(worksheet)
    worksheet.set_size(2, 8)

    self.mox.ReplayAll()

    self.collection.create_spreadsheet('Cinamon', rows=2, cols=8)
